/* Copyright (c) Jython Developers */
package org.python.modules.bz2;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

import org.python.core.ArgParser;
import org.python.core.Py;
import org.python.core.PyObject;
import org.python.core.PyString;
import org.python.core.PyType;
import org.python.core.util.StringUtil;
import org.python.expose.ExposedGet;
import org.python.expose.ExposedMethod;
import org.python.expose.ExposedNew;
import org.python.expose.ExposedType;
import org.python.modules.bz2.io.BZip2CompressorOutputStream;

/**
 * A streaming bz2 compressor.
 */
@ExposedType(name = "bz2.BZ2Compressor")
public class BZ2Compressor extends PyObject {

    @ExposedGet
    public static final PyObject __doc__ = new PyString(
        "BZ2Compressor([compresslevel=9]) -> compressor object\n" +
        "\n" +
        "Create a new compressor object. This object may be used to compress\n" +
        "data sequentially. If you want to compress data in one shot, use the\n" +
        "compress() function instead. The compresslevel parameter, if given,\n" +
        "must be a number between 1 and 9.\n");

    /** The size XXX */
    public static final int BUFFER_SIZE = 8192;

    private OutputStream compressor;

    private ByteArrayOutputStream out;

    private boolean running = true;

    public BZ2Compressor() {
        super();
    }

    public BZ2Compressor(PyType subType) {
        super(subType);
    }

    public BZ2Compressor(int compresslevel) {
        super();
        BZ2Compressor___init__(compresslevel);
    }

    @ExposedNew
    @ExposedMethod
    final void BZ2Compressor___init__(PyObject[] args, String[] kwds) {
        ArgParser ap = new ArgParser("__init__", args, kwds, new String[] {"compresslevel"});
        int compresslevel = ap.getInt(0, 9);
        BZ2Compressor___init__(compresslevel);
    }

    private void BZ2Compressor___init__(int compresslevel) {
        if (compresslevel < 1 || compresslevel > 9) {
            throw Py.ValueError("compresslevel must be between 1 and 9");
        }
        out = new ByteArrayOutputStream(BUFFER_SIZE);
        try {
            compressor = new BZip2CompressorOutputStream(out, compresslevel);
        } catch (IOException ioe) {
            throw Py.IOError(ioe);
        }
    }

    public String compress(String data) {
        return BZ2Compressor_compress(data);
    }

    @ExposedMethod(doc = "XXX")
    final synchronized String BZ2Compressor_compress(String data) {
        ensureRunning();

        byte[] bytes = StringUtil.toBytes(data);
        try {
            compressor.write(bytes, 0, bytes.length);
        } catch (IOException ioe) {
            throw Py.IOError(ioe);
        }
        String compressed = getOutput();
        out.reset();
        return compressed;
    }

    public String flush() {
        return BZ2Compressor_flush();
    }

    @ExposedMethod(doc = "XXX")
    final synchronized String BZ2Compressor_flush() {
        ensureRunning();
        running = false;

        try {
            compressor.close();
        } catch (IOException ioe) {
            throw Py.IOError(ioe);
        }
        String compressed = getOutput();
        // Garbage collect the streams
        compressor = null;
        out = null;
        return compressed;
    }

    private void ensureRunning() {
        if (!running) {
            throw Py.ValueError("this object was already flushed");
        }
    }

    private String getOutput() {
        try {
            return out.toString("ISO-8859-1");
        } catch (UnsupportedEncodingException uee) {
            throw Py.SystemError("Java couldn't find the ISO-8859-1 encoding");
        }
    }

    @Override
    public String toString() {
        return BZ2Compressor_toString();
    }

    @ExposedMethod(names = {"__str__", "__repr__"}, doc = "XXX")
    final String BZ2Compressor_toString() {
        return String.format("<bz2.BZ2Compressor object at %s>", Py.idstr(this));
    }
}
